@@ -19,10 +19,12 @@
 #define BSON_HAVE_STRNLEN 1
 #define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
 #define BSON_HAVE_ATOMIC_64_ADD_AND_FETCH 1
+#define BSON_HAVE_SYSCALL_TID 1
 #endif
 
 /* Fix for snow leopard */
 #ifdef __APPLE__
+#define BSON_HAVE_REALLOCF 1
 #include <Availability.h>
 #ifndef MAC_OS_X_VERSION_10_8
 #undef BSON_HAVE_STRNLEN
@@ -31,8 +33,8 @@
 #endif
 
 /*
- * Define to 1 if you have stdbool.h
- */
+* Define to 1 if you have stdbool.h
+*/
 #define BSON_HAVE_STDBOOL_H 1
 #if BSON_HAVE_STDBOOL_H != 1
 # undef BSON_HAVE_STDBOOL_H
@@ -40,37 +42,38 @@
 
 
 /*
- * Define to 1 for POSIX-like systems, 2 for Windows.
- */
+* Define to 1 for POSIX-like systems, 2 for Windows.
+*/
 #ifdef _WIN32
 #define BSON_OS 2
+#undef BSON_HAVE_SYSCALL_TID
 #else
 #define BSON_OS 1
 #define BSON_HAVE_CLOCK_GETTIME 1
 #endif
 
 /*
- * Define to 1 if we have access to GCC 32-bit atomic builtins.
- * While this requires GCC 4.1+ in most cases, it is also architecture
- * dependent. For example, some PPC or ARM systems may not have it even
- * if it is a recent GCC version.
- */
+* Define to 1 if we have access to GCC 32-bit atomic builtins.
+* While this requires GCC 4.1+ in most cases, it is also architecture
+* dependent. For example, some PPC or ARM systems may not have it even
+* if it is a recent GCC version.
+*/
 #if BSON_HAVE_ATOMIC_32_ADD_AND_FETCH != 1
 # undef BSON_HAVE_ATOMIC_32_ADD_AND_FETCH
 #endif
 
 /*
- * Similarly, define to 1 if we have access to GCC 64-bit atomic builtins.
- */
+* Similarly, define to 1 if we have access to GCC 64-bit atomic builtins.
+*/
 #if BSON_HAVE_ATOMIC_64_ADD_AND_FETCH != 1
 # undef BSON_HAVE_ATOMIC_64_ADD_AND_FETCH
 #endif
 
 
 /*
- * Define to 1 if your system requires {} around PTHREAD_ONCE_INIT.
- * This is typically just Solaris 8-10.
- */
+* Define to 1 if your system requires {} around PTHREAD_ONCE_INIT.
+* This is typically just Solaris 8-10.
+*/
 #define BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES 0
 #if BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES != 1
 # undef BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES
@@ -78,24 +81,24 @@
 
 
 /*
- * Define to 1 if you have clock_gettime() available.
- */
+* Define to 1 if you have clock_gettime() available.
+*/
 #if BSON_HAVE_CLOCK_GETTIME != 1
 # undef BSON_HAVE_CLOCK_GETTIME
 #endif
 
 
 /*
- * Define to 1 if you have strnlen available on your platform.
- */
+* Define to 1 if you have strnlen available on your platform.
+*/
 #if BSON_HAVE_STRNLEN != 1
 # undef BSON_HAVE_STRNLEN
 #endif
 
 
 /*
- * Define to 1 if you have snprintf available on your platform.
- */
+* Define to 1 if you have snprintf available on your platform.
+*/
 #define BSON_HAVE_SNPRINTF 1
 #if BSON_HAVE_SNPRINTF != 1
 # undef BSON_HAVE_SNPRINTF
@@ -103,16 +106,16 @@
 
 
 /*
-* Define to 1 if you have _set_output_format (VS2013 and older).
-*/
+ * Define to 1 if you have _set_output_format (VS2013 and older).
+ */
 #define BSON_NEEDS_SET_OUTPUT_FORMAT 0
 #if BSON_NEEDS_SET_OUTPUT_FORMAT != 1
 # undef BSON_NEEDS_SET_OUTPUT_FORMAT
 #endif
 
 /*
-* Define to 1 if you have struct timespec available on your platform.
-*/
+ * Define to 1 if you have struct timespec available on your platform.
+ */
 #define BSON_HAVE_TIMESPEC 1
 #if BSON_HAVE_TIMESPEC != 1
 # undef BSON_HAVE_TIMESPEC
@@ -120,13 +123,20 @@
 
 
 /*
-* Define to 1 if you want extra aligned types in libbson
-* Moved to this in Makevars because it doesn't work well with mingw
-* #if BSON_EXTRA_ALIGN != 1
-* # undef BSON_EXTRA_ALIGN
-* #endif
+* Define to 1 if _Decimal128 (BID format) is available on your platform.
 */
+#define BSON_HAVE_DECIMAL128 0
+#if BSON_HAVE_DECIMAL128 != 1
+# undef BSON_HAVE_DECIMAL128
+#endif
 
 
+/*
+* Define to 1 to support experimental future BSON or MongoDB features.
+*/
+#define BSON_EXPERIMENTAL_FEATURES 0
+#if BSON_EXPERIMENTAL_FEATURES != 1
+# undef BSON_EXPERIMENTAL_FEATURES
+#endif
 
 #endif /* BSON_CONFIG_H */
