@@ -42,11 +42,11 @@
  */
 
 static BSON_INLINE void
-_bson_utf8_get_sequence (const char *utf8,       /* IN */
-                         uint8_t    *seq_length, /* OUT */
-                         uint8_t    *first_mask) /* OUT */
+_bson_utf8_get_sequence (const char *utf8,    /* IN */
+                         uint8_t *seq_length, /* OUT */
+                         uint8_t *first_mask) /* OUT */
 {
-   unsigned char c = *(const unsigned char *)utf8;
+   unsigned char c = *(const unsigned char *) utf8;
    uint8_t m;
    uint8_t n;
 
@@ -115,9 +115,9 @@ _bson_utf8_get_sequence (const char *utf8,       /* IN */
  */
 
 bool
-bson_utf8_validate (const char *utf8,       /* IN */
-                    size_t      utf8_len,   /* IN */
-                    bool        allow_null) /* IN */
+bson_utf8_validate (const char *utf8, /* IN */
+                    size_t utf8_len,  /* IN */
+                    bool allow_null)  /* IN */
 {
    bson_unichar_t c;
    uint8_t first_mask;
@@ -125,7 +125,7 @@ bson_utf8_validate (const char *utf8,       /* IN */
    unsigned i;
    unsigned j;
 
-   bson_return_val_if_fail (utf8, false);
+   BSON_ASSERT (utf8);
 
    for (i = 0; i < utf8_len; i += seq_length) {
       _bson_utf8_get_sequence (&utf8[i], &seq_length, &first_mask);
@@ -148,13 +148,13 @@ bson_utf8_validate (const char *utf8,       /* IN */
        * Also calculate the next char as a unichar so we can
        * check code ranges for non-shortest form.
        */
-      c = utf8 [i] & first_mask;
+      c = utf8[i] & first_mask;
 
       /*
        * Check the high-bits for each additional sequence byte.
        */
       for (j = i + 1; j < (i + seq_length); j++) {
-         c = (c << 6) | (utf8 [j] & 0x3F);
+         c = (c << 6) | (utf8[j] & 0x3F);
          if ((utf8[j] & 0xC0) != 0x80) {
             return false;
          }
@@ -261,15 +261,15 @@ bson_utf8_validate (const char *utf8,       /* IN */
  */
 
 char *
-bson_utf8_escape_for_json (const char *utf8,     /* IN */
-                           ssize_t     utf8_len) /* IN */
+bson_utf8_escape_for_json (const char *utf8, /* IN */
+                           ssize_t utf8_len) /* IN */
 {
    bson_unichar_t c;
    bson_string_t *str;
    bool length_provided = true;
    const char *end;
 
-   bson_return_val_if_fail (utf8, NULL);
+   BSON_ASSERT (utf8);
 
    str = bson_string_new (NULL);
 
@@ -307,7 +307,7 @@ bson_utf8_escape_for_json (const char *utf8,     /* IN */
          break;
       default:
          if (c < ' ') {
-            bson_string_append_printf (str, "\\u%04u", (unsigned)c);
+            bson_string_append_printf (str, "\\u%04u", (unsigned) c);
          } else {
             bson_string_append_unichar (str, c);
          }
@@ -359,7 +359,7 @@ bson_utf8_get_char (const char *utf8) /* IN */
    uint8_t num;
    int i;
 
-   bson_return_val_if_fail (utf8, -1);
+   BSON_ASSERT (utf8);
 
    _bson_utf8_get_sequence (utf8, &num, &mask);
    c = (*utf8) & mask;
@@ -398,7 +398,7 @@ bson_utf8_next_char (const char *utf8) /* IN */
    uint8_t mask;
    uint8_t num;
 
-   bson_return_val_if_fail (utf8, NULL);
+   BSON_ASSERT (utf8);
 
    _bson_utf8_get_sequence (utf8, &num, &mask);
 
@@ -430,13 +430,12 @@ bson_utf8_next_char (const char *utf8) /* IN */
  */
 
 void
-bson_utf8_from_unichar (
-      bson_unichar_t  unichar,                               /* IN */
-      char            utf8[BSON_ENSURE_ARRAY_PARAM_SIZE(6)], /* OUT */
-      uint32_t       *len)                                   /* OUT */
+bson_utf8_from_unichar (bson_unichar_t unichar,                      /* IN */
+                        char utf8[BSON_ENSURE_ARRAY_PARAM_SIZE (6)], /* OUT */
+                        uint32_t *len)                               /* OUT */
 {
-   bson_return_if_fail (utf8);
-   bson_return_if_fail (len);
+   BSON_ASSERT (utf8);
+   BSON_ASSERT (len);
 
    if (unichar <= 0x7F) {
       utf8[0] = unichar;
@@ -444,25 +443,25 @@ bson_utf8_from_unichar (
    } else if (unichar <= 0x7FF) {
       *len = 2;
       utf8[0] = 0xC0 | ((unichar >> 6) & 0x3F);
-      utf8[1] = 0x80 | ((unichar) & 0x3F);
+      utf8[1] = 0x80 | ((unichar) &0x3F);
    } else if (unichar <= 0xFFFF) {
       *len = 3;
       utf8[0] = 0xE0 | ((unichar >> 12) & 0xF);
       utf8[1] = 0x80 | ((unichar >> 6) & 0x3F);
-      utf8[2] = 0x80 | ((unichar) & 0x3F);
+      utf8[2] = 0x80 | ((unichar) &0x3F);
    } else if (unichar <= 0x1FFFFF) {
       *len = 4;
       utf8[0] = 0xF0 | ((unichar >> 18) & 0x7);
       utf8[1] = 0x80 | ((unichar >> 12) & 0x3F);
       utf8[2] = 0x80 | ((unichar >> 6) & 0x3F);
-      utf8[3] = 0x80 | ((unichar) & 0x3F);
+      utf8[3] = 0x80 | ((unichar) &0x3F);
    } else if (unichar <= 0x3FFFFFF) {
       *len = 5;
       utf8[0] = 0xF8 | ((unichar >> 24) & 0x3);
       utf8[1] = 0x80 | ((unichar >> 18) & 0x3F);
       utf8[2] = 0x80 | ((unichar >> 12) & 0x3F);
       utf8[3] = 0x80 | ((unichar >> 6) & 0x3F);
-      utf8[4] = 0x80 | ((unichar) & 0x3F);
+      utf8[4] = 0x80 | ((unichar) &0x3F);
    } else if (unichar <= 0x7FFFFFFF) {
       *len = 6;
       utf8[0] = 0xFC | ((unichar >> 31) & 0x1);
@@ -470,7 +469,7 @@ bson_utf8_from_unichar (
       utf8[2] = 0x80 | ((unichar >> 19) & 0x3F);
       utf8[3] = 0x80 | ((unichar >> 13) & 0x3F);
       utf8[4] = 0x80 | ((unichar >> 7) & 0x3F);
-      utf8[5] = 0x80 | ((unichar) & 0x1);
+      utf8[5] = 0x80 | ((unichar) &0x1);
    } else {
       *len = 0;
    }
