@@ -15,7 +15,7 @@
  */
 
 
-#if !defined (MONGOC_INSIDE) && !defined (MONGOC_COMPILATION)
+#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
 #error "Only <mongoc.h> can be included directly."
 #endif
 
@@ -29,9 +29,12 @@
 
 BSON_BEGIN_DECLS
 
-void mongoc_rand_seed(const void* buf, int num);
-void mongoc_rand_add(const void* buf, int num, double entropy);
-int mongoc_rand_status(void);
+BSON_EXPORT (void)
+mongoc_rand_seed (const void *buf, int num);
+BSON_EXPORT (void)
+mongoc_rand_add (const void *buf, int num, double entropy);
+BSON_EXPORT (int)
+mongoc_rand_status (void);
 
 BSON_END_DECLS
 
